<?php
namespace WPCommander\Models;

defined( 'ABSPATH' ) || exit;

class FoldersModel {
    const COMMON_FOLDERS = -1;
    const PERSONAL_FOLDERS = -2;

    public static function getCurrentType() {
        return 'attachment';
    }

    public static function getFolderAccessTypeTitle( $type ) {
        switch ( $type ) {
            case self::COMMON_FOLDERS: {
                return __( "Common Folders", 'wpcommander' );
            } break;
            case self::PERSONAL_FOLDERS: {
                return __( "Personal Folders", 'wpcommander' );
            } break;
        }
        return null;
    }

    public static function getFolderAccessTypes() {
        $data['items'] = [];
        $data['items'][] = [ 'id' => self::COMMON_FOLDERS, 'title' => self::getFolderAccessTypeTitle( self::COMMON_FOLDERS )  ];
        $data['items'][] = [ 'id' => self::PERSONAL_FOLDERS, 'title' => self::getFolderAccessTypeTitle( self::PERSONAL_FOLDERS ) ];
        $data['total'] = count( $data['items'] );

        return $data;
    }

    public static function getFolderAccessType() {
        return ConfigModel::get('folder_access_type');
    }

    private static function sortTree( $items, &$out=[], $parent=0, $level=0) {
        foreach ( $items as $key => $item ) {
            if ( $item['parent'] == $parent ) {
                $item['level'] = $level;
                $out[] = $item;
                unset( $items[ $key ] );
                self::sortTree( $items, $out, $item['id'], $level + 1 );
            }
        }
        return $out;
    }

    private static function convertFlatToTree( &$folders, $parent = 0 ) {
        $branch = [];
        foreach( $folders as $key => $folder ) {
            if ( $folder['parent'] == $parent ) {
                $children = self::convertFlatToTree( $folders, $folder['id'] );

                if ( $children ) {
                    $folder['items'] = $children;
                }

                if ( array_key_exists( 'items', $folder ) && $folder['items'] ) {
                    $branch[] = [
                        'id' => $folder['id'],
                        'own' => $folder['owner'],
                        'title' => $folder['title'],
                        'items' => $folder['items'],
                        'color' => $folder['color'],
                        'count' => intval( $folder['count'] )
                    ];
                } else {
                    $branch[] = [
                        'id' => $folder['id'],
                        'own' => $folder['owner'],
                        'title' => $folder['title'],
                        'color' => $folder['color'],
                        'count' => intval( $folder['count'] )
                    ];
                }

                unset( $folders[$key] );
            }
        }
        return $branch;
    }

    private static function getChildFolders( $parent, &$out ) {
        global $wpdb;
        $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';

        $sql = $wpdb->prepare( "SELECT id FROM {$tableFolders} WHERE parent=%d ORDER BY ord", $parent );
        $items = $wpdb->get_results( $sql, 'ARRAY_A' );

        if ( $items ) {
            foreach ( $items as $item ) {
                $out[] = $item['id'];
                self::getChildFolders( $item['id'], $out );
            }
        }
    }

    private static function getParentAndChildFoldersForCopy( $parent, $level, &$out ) {
        global $wpdb;
        $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';

        $sql = $wpdb->prepare( "SELECT id, owner, title, type, color, ord FROM {$tableFolders} WHERE id=%d ORDER BY ord", $parent );
        $item = $wpdb->get_row( $sql, 'ARRAY_A' );

        if ( isset( $item ) ) {
            $item['level'] = $level;
            $out[] = $item;

            $sql = $wpdb->prepare( "SELECT id, owner, title, type, color, ord FROM {$tableFolders} WHERE parent=%d ORDER BY ord", $parent );
            $items = $wpdb->get_results( $sql, 'ARRAY_A' );

            if ( $items ) {
                foreach( $items as $item ) {
                    self::getParentAndChildFoldersForCopy( $item['id'], $level + 1, $out );
                }
            }
        }
    }

    public static function getFolders( $type ) {
        if ( UserModel::hasAccess() ) {
            global $wpdb;
            $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';

            switch( self::getFolderAccessType() ) {
                case self::COMMON_FOLDERS: {
                        $sql = $wpdb->prepare("
                            SELECT id, title, parent, color, owner, count
                            FROM {$tableFolders}
                            WHERE type=%s AND owner=0
                            ORDER BY ord, created",
                            $type
                        );
                        $folders = $wpdb->get_results($sql, 'ARRAY_A');

                        if (!$wpdb->last_error) {
                            return self::convertFlatToTree($folders);
                        }
                } break;
                case self::PERSONAL_FOLDERS: {
                        $sql = $wpdb->prepare("
                            SELECT id, title, parent, color, owner, count
                            FROM {$tableFolders}
                            WHERE type=%s AND owner=%d
                            ORDER BY ord, created",
                            $type, get_current_user_id()
                        );
                        $folders = $wpdb->get_results($sql, 'ARRAY_A');

                        if (!$wpdb->last_error) {
                            return self::convertFlatToTree($folders);
                        }
                } break;
            }
        }
        return null;
    }

    public static function createFolders( $type, $parent, $names, $color ) {
        if ( UserModel::hasAccess() ) {
            $folders = [];

            foreach ($names as $name) {
                global $wpdb;
                $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
                $owner = self::getFolderAccessType() == self::COMMON_FOLDERS ? 0 : get_current_user_id();

                $wpdb->insert(
                    $tableFolders,
                    [
                        'parent' => $parent,
                        'type' => $type,
                        'owner' => $owner,
                        'title' => $name,
                        'color' => $color,
                        'ord' => PHP_INT_MAX,
                        'created_by' => get_current_user_id(),
                        'modified_by' => get_current_user_id(),
                        'created' => current_time( 'mysql', 1 ),
                        'modified' => current_time( 'mysql', 1 )
                    ]
                );

                if ( !$wpdb->last_error ) {
                    $folders[] = [
                        'id' => strval( $wpdb->insert_id ),
                        'title' => $name,
                        'color' => $color
                    ];
                } else {
                    break;
                }
            }

            if ( !$wpdb->last_error ) {
                return $folders;
            }
        }
        return null;
    }

    public static function updateFolders( $type, $action, $ids, $attrs ) {
        if ( UserModel::hasAccess() ) {
            if ( is_array( $ids ) && count( $ids ) > 0 ) {
                global $wpdb;
                $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';

                $folders_to_edit = [];
                $ids = implode( ',', array_map( 'intval', $ids ) );

                switch( self::getFolderAccessType() ) {
                    case self::COMMON_FOLDERS: {
                        $sql = $wpdb->prepare( "SELECT id FROM {$tableFolders} WHERE type=%s AND owner=0 AND id IN(%1s)", $type, $ids );
                        $folders_to_edit = array_column( $wpdb->get_results( $sql, 'ARRAY_A' ), 'id' );
                    } break;
                    case self::PERSONAL_FOLDERS: {
                        $sql = $wpdb->prepare( "SELECT id FROM {$tableFolders} WHERE type=%s AND owner=%d AND id IN(%1s)", $type, get_current_user_id(), $ids );
                        $folders_to_edit = array_column( $wpdb->get_results( $sql, 'ARRAY_A' ), 'id' );
                    } break;
                }

                if ( !empty( $folders_to_edit ) ) {
                    $ids = implode( ',', array_map( 'intval', $folders_to_edit ) );

                    switch ( $action ) {
                        case 'rename': {
                            $sql = $wpdb->prepare("
                                UPDATE {$tableFolders} 
                                SET title=%s, modified_by=%d, modified=%s 
                                WHERE id IN(%1s)",
                                $attrs['name'], get_current_user_id(), current_time('mysql', 1), $ids
                            );
                            $wpdb->query( $sql );

                            if ( !$wpdb->last_error ) {
                                return $folders_to_edit;
                            }
                        } break;
                        case 'color': {
                            $sql = $wpdb->prepare("
                                UPDATE {$tableFolders} 
                                SET color=%s, modified_by=%d, modified=%s 
                                WHERE id IN(%1s)",
                                $attrs['color'], get_current_user_id(), current_time('mysql', 1), $ids
                            );
                            $wpdb->query( $sql );

                            if ( !$wpdb->last_error ) {
                                return $folders_to_edit;
                            }
                        } break;
                        case 'move': {
                            $parent = null;
                            switch( self::getFolderAccessType() ) {
                                case self::COMMON_FOLDERS: {
                                    $sql = $wpdb->prepare( "SELECT id FROM {$tableFolders} WHERE type=%s AND owner=0 AND id=%d", $type, $attrs['parent'] );
                                    $parent = $wpdb->get_var( $sql );
                                } break;
                                case self::PERSONAL_FOLDERS: {
                                    $sql = $wpdb->prepare( "SELECT id FROM {$tableFolders} WHERE type=%s AND owner=%d AND id=%d", $type, get_current_user_id(), $attrs['parent'] );
                                    $parent = $wpdb->get_var( $sql );
                                } break;
                            }
                            $parent = empty( $parent ) ? 0 : $parent;

                            $sql = $wpdb->prepare("
                                UPDATE {$tableFolders}
                                SET parent=%d, modified_by=%d, modified=%s 
                                WHERE id IN(%1s)",
                                $parent, get_current_user_id(), current_time('mysql', 1), $ids
                            );
                            $wpdb->query( $sql );

                            $sort = 0;
                            foreach ( $attrs['sorting'] as $id ) {
                                $sql = $wpdb->prepare("
                                    UPDATE {$tableFolders}
                                    SET ord=%d
                                    WHERE id=%d",
                                    $sort++, $id
                                );
                                $wpdb->query($sql);
                            }

                            if ( !$wpdb->last_error ) {
                                return $folders_to_edit;
                            }

                        } break;
                    }
                }
            }
        }
        return null;
    }

    public static function deleteFolders( $type, $ids ) {
        if ( UserModel::hasAccess() ) {
            if ( is_array( $ids ) && count( $ids ) > 0 ) {
                global $wpdb;
                $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
                $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';

                $folders_to_delete = [];
                $ids = implode( ',', array_map( 'intval', $ids ) );

                switch( self::getFolderAccessType() ) {
                    case self::COMMON_FOLDERS: {
                        $sql = $wpdb->prepare( "SELECT id FROM {$tableFolders} WHERE type=%s AND owner=0 AND id IN(%1s)", $type, $ids );
                        $folders_to_delete = array_column($wpdb->get_results($sql, 'ARRAY_A'), 'id');
                    } break;
                    case self::PERSONAL_FOLDERS: {
                        $sql = $wpdb->prepare( "SELECT id FROM {$tableFolders} WHERE type=%s AND owner=%d AND id IN(%1s)", $type, get_current_user_id(), $ids );
                        $folders_to_delete = array_column($wpdb->get_results($sql, 'ARRAY_A'), 'id');
                    } break;
                }

                if ( !empty( $folders_to_delete ) ) {
                    $folders = [];
                    foreach ( $folders_to_delete as $id ) {
                        $folders[] = $id;
                        self::getChildFolders( $id, $folders );
                    }
                    $folders = array_values( array_unique( $folders ) );

                    $ids = implode( ',', array_map( 'intval', $folders ) );

                    $sql = $wpdb->prepare( "DELETE FROM {$tableAttachments} WHERE folder_id IN(%1s)", $ids );
                    $wpdb->query( $sql );

                    $sql = $wpdb->prepare( "DELETE FROM {$tableFolders} WHERE id IN(%1s)", $ids );
                    $wpdb->query( $sql );

                    if( !$wpdb->last_error ) {
                        return $folders;
                    }
                }
            }
        }
        return null;
    }

    public static function copyFolder( $type, $src, $dst ) {
        if ( UserModel::hasAccess() ) {
            global $wpdb;
            $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';

            if ( $dst != 0 ) {
                switch ( self::getFolderAccessType() ) {
                    case self::COMMON_FOLDERS: {
                        $sql = $wpdb->prepare("SELECT id FROM {$tableFolders} WHERE type=%s AND owner=0 AND id=%d", $type, $dst);
                        $dst = $wpdb->get_var( $sql );
                    } break;
                    case self::PERSONAL_FOLDERS: {
                        $sql = $wpdb->prepare("SELECT id FROM {$tableFolders} WHERE type=%s AND owner=%d AND id=%d", $type, get_current_user_id(), $dst);
                        $dst = $wpdb->get_var( $sql );
                    } break;
                }
            }

            if ( isset( $dst ) ) {
                $folders = [];
                $folders_to_copy = [];
                self::getParentAndChildFoldersForCopy( $src, 0, $folders_to_copy );
                $folders_to_copy_length = count($folders_to_copy);

                $folder_parents[] = $dst;
                $level = 0;
                foreach ( $folders_to_copy as $key => $folder ) {
                    $wpdb->insert(
                        $tableFolders,
                        [
                            'owner' => $folder['owner'],
                            'title' => $folder['title'],
                            'parent' => $folder_parents[ $level ],
                            'type' => $folder['type'],
                            'color' => $folder['color'],
                            'created_by' => get_current_user_id(),
                            'modified_by' => get_current_user_id(),
                            'created' => current_time( 'mysql', 1 ),
                            'modified' => current_time( 'mysql', 1 ),
                            'ord' => PHP_INT_MAX
                        ]
                    );

                    if ( !$wpdb->last_error ) {
                        $folders[] = [
                            'id' => strval( $wpdb->insert_id ),
                            'title' => $folder['title'],
                            'parent' => $folder_parents[ $level ],
                            'color' => $folder['color']
                        ];
                    } else {
                        break;
                    }

                    $next = $key + 1;
                    if ( $next < $folders_to_copy_length ) {
                        $folder_next = $folders_to_copy[ $next ];

                        if ( $folder_next['level'] > $level ) {
                            array_push( $folder_parents, strval( $wpdb->insert_id ) );
                            $level++;
                        } else if( $folder_next['level'] < $level ) {
                            array_pop( $folder_parents );
                            $level--;
                        }
                    }
                }

                return $folders;
            }
        }
        return null;
    }

    public static function attachToFolder( $type, $id, $attachments ) {
        if ( UserModel::hasAccess() ) {
            global $wpdb;
            $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
            $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';

            if (  is_array( $attachments ) && count( $attachments ) > 0) {
                $folder_dest = null;
                if( !( $id == -1 || $id == -2 ) ) {
                    switch( self::getFolderAccessType() ) {
                        case self::COMMON_FOLDERS: {
                            $sql = $wpdb->prepare( "SELECT id FROM {$tableFolders} WHERE type=%s AND owner=0 AND id=%d", $type, $id );
                            $folder_dest = $wpdb->get_var( $sql );
                        } break;
                        case self::PERSONAL_FOLDERS: {
                            $sql = $wpdb->prepare( "SELECT id FROM {$tableFolders} WHERE type=%s AND owner=%d AND id=%d", $type, get_current_user_id(), $id );
                            $folder_dest = $wpdb->get_var( $sql );
                        } break;
                    }
                }

                // folders to refresh after the update
                $folders_to_refresh = [];
                $ids = implode( ',', array_map( 'intval', $attachments ) );
                $owner = self::getFolderAccessType() == self::COMMON_FOLDERS ? 0 : get_current_user_id();

                $sql = $wpdb->prepare("
                    SELECT DISTINCT A.folder_id as id 
                    FROM {$tableAttachments} AS A 
                    LEFT JOIN {$tableFolders} AS F 
                    ON F.id=A.folder_id AND F.owner=%d
                    WHERE A.attachment_id IN(%1s)",
                    $owner, $ids
                );
                $folders_to_refresh = array_column( $wpdb->get_results( $sql, 'ARRAY_A' ), 'id' );
                $folders_to_refresh[] = $folder_dest;

                // delete previous attachments
                $sql = $wpdb->prepare("
                    DELETE A.* 
                    FROM {$tableAttachments} AS A 
                    LEFT JOIN {$tableFolders} AS F 
                    ON F.id=A.folder_id AND F.owner=%d
                    WHERE A.attachment_id IN(%1s)",
                    $owner, $ids
                );
                $wpdb->query( $sql );

                // add new attachments
                if ( isset( $folder_dest ) && !( $folder_dest == -1 || $folder_dest == -2 ) ) {
                    foreach( $attachments as $attachment ) {
                        $wpdb->insert(
                            $tableAttachments,
                            [
                                'folder_id' => $folder_dest,
                                'attachment_id' => $attachment
                            ]
                        );
                    }
                }

                // update the attachment count
                if ( !empty( $folders_to_refresh ) ) {
                    $ids = implode(',', array_map( 'intval', $folders_to_refresh ) );
                    $sql = $wpdb->prepare("
                        UPDATE {$tableFolders} AS F 
                        SET count = (SELECT COUNT(folder_id) FROM {$tableAttachments} AS A WHERE A.folder_id=F.id) 
                        WHERE id IN(%1s)",
                        $ids
                    );
                    $wpdb->query($sql);
                }

                if ( !$wpdb->last_error ) {
                    return $folders_to_refresh;
                }
            }
        }
        return null;
    }

    public static function updateAttachmentCounters() {
        if ( UserModel::hasAccess() ) {
            global $wpdb;
            $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
            $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';

            $sql = "DELETE FROM {$tableAttachments} WHERE attachment_id NOT IN (SELECT ID FROM {$wpdb->posts})";
            $wpdb->query( $sql );

            $sql = "UPDATE {$tableFolders} AS F SET count = (SELECT COUNT(folder_id) FROM {$tableAttachments} AS A WHERE A.folder_id=F.id)";
            $wpdb->query( $sql );

            if ( !$wpdb->last_error ) {
                return true;
            }
        }
        return false;
    }

    public static function getAttachmentCounters( $type, $ids ) {
        global $wpdb;
        $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
        $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';

        $owner = self::getFolderAccessType() == self::COMMON_FOLDERS ? 0 : get_current_user_id();

        // folders to refresh after the update
        $folders_to_refresh = [];
        if ( is_array( $ids ) && count( $ids ) > 0 ) {
            $ids = implode( ',', array_map( 'intval', $ids ) );

            $sql = $wpdb->prepare("
                SELECT id, count
                FROM {$tableFolders} 
                WHERE type=%s AND owner=%d AND id IN(%1s)",
                $type, $owner, $ids
            );
            $folders_to_refresh = $wpdb->get_results( $sql, 'ARRAY_A' );
        }

        switch ( $type ) {
            case 'users': {
                $sql = "SELECT COUNT(id) FROM {$wpdb->users}";
                $folders_to_refresh[] = [ 'id' => '-1', 'count' => $wpdb->get_var( $sql ) ];

                $sql = $wpdb->prepare("
                    SELECT COUNT(id)
                    FROM {$wpdb->users}
                    WHERE id NOT IN (
                        SELECT attachment_id 
                        FROM {$tableAttachments} AS A 
                        LEFT JOIN {$tableFolders} AS F 
                        ON F.id = A.folder_id 
                        WHERE F.type=%s AND F.owner=%d
                    )",
                    $type, $owner
                );
                $folders_to_refresh[] = [ 'id' => '-2', 'count' => $wpdb->get_var( $sql ) ];
            } break;
            default: {
                $sql = $wpdb->prepare("
                    SELECT COUNT(id)
                    FROM {$wpdb->posts}
                    WHERE post_type=%s",
                    $type
                );
                $folders_to_refresh[] = [ 'id' => '-1', 'count' => $wpdb->get_var( $sql ) ];

                $sql = $wpdb->prepare("
                    SELECT COUNT(id)
                    FROM {$wpdb->posts}
                    WHERE post_type=%s AND id NOT IN (
                        SELECT attachment_id 
                        FROM {$tableAttachments} AS A 
                        LEFT JOIN {$tableFolders} AS F 
                        ON F.id = A.folder_id 
                        WHERE F.type=%s AND F.owner=%d
                    )",
                    $type, $type, $owner
                );
                $folders_to_refresh[] = [ 'id' => '-2', 'count' => $wpdb->get_var( $sql ) ];
            } break;
        }

        return $folders_to_refresh;
    }

    public static function getAttachmentFiles( $id ) {
        global $wpdb;
        $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';

        $sql = "SELECT attachment_id FROM {$tableAttachments} WHERE folder_id = %d";
        $ids = $wpdb->get_col( $wpdb->prepare( $sql, $id ) );

        $files = [];
        if( count( $ids ) > 0 ) {
            $ids = implode( ',', $ids );
            $sql = $wpdb->prepare("SELECT meta_value FROM {$wpdb->postmeta} WHERE meta_key = '_wp_attached_file' AND post_id IN(%1s)", $ids );
            $files = $wpdb->get_col($sql);
        }

        return $files;
    }

    public static function exportCSV() {
        global $wpdb;
        $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
        $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';

        $sql = "SELECT id, owner, title, parent, type, color, ord FROM {$tableFolders} ORDER BY ord, created";
        $folders = $wpdb->get_results( $sql, 'ARRAY_A' );

        foreach ($folders as $key => $folder) {
            $sql = "SELECT attachment_id FROM {$tableAttachments} WHERE folder_id = %d";
            $folders[$key]['attachments'] = $wpdb->get_col( $wpdb->prepare( $sql, $folder['id'] ) );
        }

        return $folders;
    }

    public static function importCSV( $file, $clear = false, $attachments = false ) {
        $folders = [];
        $columns = [];
        $handle = \fopen( $file, 'r' );

        if ( $handle !== false ) {
            $count = 1;
            while ( true ) {
                $row = fgetcsv( $handle, 0 );
                if ( $count === 1 ) {
                    $columns = $row;
                    $count++;
                    continue;
                }
                if ( $row === false ) {
                    break;
                }
                foreach ( $columns as $key => $col ) {
                    $data[ $col ] = $row[ $key ];
                }
                $folders[] = $data;
            }
        }
        \fclose( $handle );

        return self::importFolders( $folders, $clear, $attachments );
    }

    public static function importFolders( $folders, $clear = false, $attachments = false ) {
        global $wpdb;
        $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
        $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';

        if ( $clear ) {
            $wpdb->query( "DELETE FROM {$tableFolders}" );
            $wpdb->query( "DELETE FROM {$tableAttachments}" );
        }

        $out = [];
        self::sortTree( $folders, $out );
        $folders = $out;

        foreach ( $folders as $key => $folder ) {
            $wpdb->insert(
                $tableFolders,
                [
                    'owner' => $folder['owner'],
                    'title' => $folder['title'],
                    'parent' => $folders[ $key ]['parent'],
                    'type' => $folder['type'],
                    'color' => $folder['color'],
                    'created_by' => get_current_user_id(),
                    'modified_by' => get_current_user_id(),
                    'created' => current_time( 'mysql', 1 ),
                    'modified' => current_time( 'mysql', 1 ),
                    'ord' => $folder['ord']
                ]
            );

            if ( !$wpdb->last_error ) {
                $id = strval( $wpdb->insert_id );

                // find the old identifier of the parent and change it to the new value
                for ( $i = $key + 1; $i < count( $folders ); $i++ ) {
                    if ( $folders[ $i ]['parent'] == $folders[ $key ]['id'] ) {
                        $folders[ $i ]['parent'] = $id;
                    }
                }

                $folders[ $key ]['id'] = $id;

                if ( $attachments && $folder['attachments'] !== '' ) {
                    if ( is_array( $folder['attachments'] ) ) {
                        self::importAttachments( $id, $folder['type'], $folder['owner'], $folder['attachments'] );
                    } else {
                        self::importAttachments( $id, $folder['type'], $folder['owner'], explode('|', $folder['attachments'] ) );
                    }
                }
            } else {
                return null;
            }
        }

        self::updateAttachmentCounters();

        return $folders;
    }

    public static function importAttachments( $id, $type, $owner, $attachments ) {
        if ( is_array( $attachments ) && count( $attachments ) > 0 ) {
            global $wpdb;
            $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
            $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';

            $ids = implode( ',', array_map( 'intval', $attachments ) );

            // folders to refresh after the update
            $sql = $wpdb->prepare("
                SELECT DISTINCT A.folder_id as id 
                FROM {$tableAttachments} AS A 
                LEFT JOIN {$tableFolders} AS F 
                ON F.id=A.folder_id AND F.type=%s AND F.owner=%d
                WHERE A.attachment_id IN(%1s)",
                $type, $owner, $ids
            );
            $folders_to_refresh = array_column( $wpdb->get_results( $sql, 'ARRAY_A' ), 'id' );

            // delete previous attachments
            $sql = $wpdb->prepare("
                DELETE A.* 
                FROM {$tableAttachments} AS A 
                LEFT JOIN {$tableFolders} AS F 
                ON F.id=A.folder_id AND F.type=%s AND F.owner=%d
                WHERE A.attachment_id IN(%1s)",
                $type, $owner, $ids
            );
            $wpdb->query( $sql );

            // add new attachments
            foreach ( $attachments as $attachment ) {
                $wpdb->insert(
                    $tableAttachments,
                    [
                        'folder_id' => $id,
                        'attachment_id' => $attachment
                    ]
                );
            }

            // update the attachment count
            $sql = $wpdb->prepare("
                UPDATE {$tableFolders} AS F 
                SET count = (SELECT COUNT(folder_id) FROM {$tableAttachments} AS A WHERE A.folder_id=F.id) 
                WHERE F.id=%d",
                $id
            );
            $wpdb->query( $sql );

            // update the attachment count
            if ( !empty( $folders_to_refresh ) ) {
                $ids = implode( ',', array_map( 'intval', $folders_to_refresh ) );
                $sql = $wpdb->prepare("
                    UPDATE {$tableFolders} AS F 
                    SET count = (SELECT COUNT(folder_id) FROM {$tableAttachments} AS A WHERE A.folder_id=F.id) 
                    WHERE id IN(%1s)",
                    $ids
                );
                $wpdb->query( $sql );
            }
        }
    }
}
