<?php
namespace WPCommander\System;

defined( 'ABSPATH' ) || exit;

use WPCommander\Models\FoldersModel;
use WPCommander\Models\HelperModel;
use WPCommander\Models\ConfigModel;
use WPCommander\Models\UserModel;

class Folders {
    private $access = false;

    public function __construct() {
        add_action( 'init', [ $this, 'init' ] );
    }

    public function init() {
        $config = ConfigModel::get();
        if ( array_key_exists('infinite_scrolling', $config) && $config['infinite_scrolling'] ) {
            add_filter( 'media_library_infinite_scrolling', '__return_true' );
        }

        add_action( 'admin_enqueue_scripts', [ $this, 'enqueueScripts' ] );
        add_action( 'wp_enqueue_scripts', [ $this, 'enqueueScripts' ] );

        add_action( 'pre_get_posts', [ $this, 'redirectToMain' ] );
        add_action( 'delete_post', [ $this, 'deletePost' ] );
        add_action( 'add_attachment', [ $this, 'addAttachment' ] );
        add_filter( 'posts_clauses', [ $this, 'postsСlauses' ] );
        add_filter( 'pre_user_query', [ $this, 'preUserQuery']);
    }

    public function enqueueScripts() {
        if ( UserModel::hasAccess() ) {
            add_action( 'admin_head', [ $this, 'adminHead' ] );

            wp_enqueue_script( 'wpcommander-cookie', WPCOMMANDER_PLUGIN_URL . 'assets/vendor/cookie/cookie.js', [], WPCOMMANDER_PLUGIN_VERSION, false );
            wp_enqueue_script( 'wpcommander-url', WPCOMMANDER_PLUGIN_URL . 'assets/vendor/url/url.js', [], WPCOMMANDER_PLUGIN_VERSION, false );

            wp_enqueue_style( 'wpcommander-colorpicker', WPCOMMANDER_PLUGIN_URL . 'assets/css/colorpicker.css', [], WPCOMMANDER_PLUGIN_VERSION );
            wp_enqueue_script( 'wpcommander-colorpicker', WPCOMMANDER_PLUGIN_URL . 'assets/js/colorpicker.js', ['jquery'], WPCOMMANDER_PLUGIN_VERSION, false );

            wp_enqueue_style( 'wpcommander-notify', WPCOMMANDER_PLUGIN_URL . 'assets/css/notify.css', [], WPCOMMANDER_PLUGIN_VERSION );
            wp_enqueue_script( 'wpcommander-notify', WPCOMMANDER_PLUGIN_URL . 'assets/js/notify.js', ['jquery'], WPCOMMANDER_PLUGIN_VERSION, false );

            wp_enqueue_script( 'wpcommander-tree', WPCOMMANDER_PLUGIN_URL . 'assets/js/tree.js', ['jquery'], WPCOMMANDER_PLUGIN_VERSION, false );

            wp_enqueue_style( 'wpcommander-sidebar', WPCOMMANDER_PLUGIN_URL . 'assets/css/sidebar.css', [], WPCOMMANDER_PLUGIN_VERSION );
            wp_enqueue_script( 'wpcommander-sidebar', WPCOMMANDER_PLUGIN_URL . 'assets/js/sidebar.js', ['jquery'], WPCOMMANDER_PLUGIN_VERSION, false );
            wp_localize_script( 'wpcommander-sidebar', 'wpcommander_sidebar_globals', $this->getGlobals() );
        }
    }

    public function redirectToMain() {
        $this->access = UserModel::hasAccess();
    }

    public function postsСlauses( $clauses ) {
        if ( !$this->access ) {
            return $clauses;
        }

        $type = FoldersModel::getCurrentType();
        if ( strpos( $clauses['where'], $type ) ) {
            $action = sanitize_key( filter_input( INPUT_POST, 'action', FILTER_DEFAULT ) );
            $mode = sanitize_key( filter_input( INPUT_POST, 'wpcommander_mode', FILTER_DEFAULT ) );
            if ( $action === 'query-attachments' && $mode !== 'grid' ) {
                return $clauses;
            }

            global $wpdb;
            $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';
            $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';

            $meta = UserModel::getMeta( $type );
            $folder = $meta['folder'];
            $folder_access_type = FoldersModel::getFolderAccessType();

            if ($folder > 0) {
                $clauses['join'] .= " LEFT JOIN {$tableAttachments} AS ATTACHMENTS ON ({$wpdb->posts}.ID = ATTACHMENTS.attachment_id)";
                $clauses['where'] = " AND (ATTACHMENTS.folder_id = $folder) " . $clauses['where'];
            } else if ($folder == -2) { // uncategorized
                switch( $folder_access_type ) {
                    case FoldersModel::COMMON_FOLDERS: {
                        $clauses['where'] .= " AND ({$wpdb->posts}.ID NOT IN (SELECT ATTACHMENTS.attachment_id FROM {$tableAttachments} AS ATTACHMENTS LEFT JOIN {$tableFolders} AS FOLDERS ON FOLDERS.id=ATTACHMENTS.folder_id WHERE FOLDERS.owner=0))";
                    } break;
                    case FoldersModel::PERSONAL_FOLDERS: {
                        $user_id = get_current_user_id();
                        $clauses['where'] .= " AND ({$wpdb->posts}.ID NOT IN (SELECT ATTACHMENTS.attachment_id FROM {$tableAttachments} AS ATTACHMENTS LEFT JOIN {$tableFolders} AS FOLDERS ON FOLDERS.id=ATTACHMENTS.folder_id WHERE FOLDERS.owner={$user_id}))";
                    } break;
                }
            }

            switch( $meta['sort']['items'] ) {
                case 'name-asc': {
                    $clauses['orderby'] = " {$wpdb->posts}.post_title ASC, " . $clauses['orderby'];
                } break;
                case 'name-desc': {
                    $clauses['orderby'] = " {$wpdb->posts}.post_title DESC, " . $clauses['orderby'];
                } break;
                case 'date-asc': {
                    $clauses['orderby'] = " {$wpdb->posts}.post_date ASC, " . $clauses['orderby'];
                } break;
                case 'date-desc': {
                    $clauses['orderby'] = " {$wpdb->posts}.post_date DESC, " . $clauses['orderby'];
                } break;
                case 'mod-asc': {
                    $clauses['orderby'] = " {$wpdb->posts}.post_modified ASC, " . $clauses['orderby'];
                } break;
                case 'mod-desc': {
                    $clauses['orderby'] = " {$wpdb->posts}.post_modified DESC, " . $clauses['orderby'];
                } break;
                case 'author-asc': {
                    $clauses['orderby'] = " {$wpdb->posts}.post_author ASC, " . $clauses['orderby'];
                } break;
                case 'author-desc': {
                    $clauses['orderby'] = " {$wpdb->posts}.post_author DESC, " . $clauses['orderby'];
                } break;
            }
        }

        return $clauses;
    }

    public function preUserQuery( $query ) {
        if ( !$this->access ) {
            return $query;
        }

        global $wpdb;
        $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';
        $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';

        $type = FoldersModel::getCurrentType();
        $meta = UserModel::getMeta( $type );
        $folder = $meta['folder'];

        if ( $folder > 0 ) {
            $query->query_from .= " LEFT JOIN {$tableAttachments} AS ATTACHMENTS ON ({$wpdb->users}.ID = ATTACHMENTS.attachment_id)";
            $query->query_where .= " AND (ATTACHMENTS.folder_id = $folder)";
        } else if($folder == -2) { // uncategorized
            switch( FoldersModel::getFolderAccessType() ) {
                case FoldersModel::COMMON_FOLDERS: {
                    $query->query_where .= " AND ($wpdb->users.ID NOT IN (SELECT ATTACHMENTS.attachment_id FROM {$tableAttachments} AS ATTACHMENTS LEFT JOIN {$tableFolders} AS FOLDERS ON FOLDERS.id=ATTACHMENTS.folder_id WHERE FOLDERS.owner=0))";
                } break;
                case FoldersModel::PERSONAL_FOLDERS: {
                    $user_id = get_current_user_id();
                    $query->query_where .= " AND ($wpdb->users.ID NOT IN (SELECT ATTACHMENTS.attachment_id FROM {$tableAttachments} AS ATTACHMENTS LEFT JOIN {$tableFolders} AS FOLDERS ON FOLDERS.id=ATTACHMENTS.folder_id WHERE FOLDERS.owner={$user_id}))";
                } break;
            }
        }

        return $query;
    }

    public function deletePost( $post_id ) {
        global $wpdb;
        $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
        $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';

        // folders to refresh after the update
        $sql = $wpdb->prepare( "SELECT DISTINCT folder_id as id FROM {$tableAttachments} WHERE attachment_id=%d", $post_id );
        $folders_to_edit = array_column( $wpdb->get_results( $sql, 'ARRAY_A' ), 'id' );

        // remove a post from the attachments table
        $sql = $wpdb->prepare("DELETE FROM {$tableAttachments} WHERE attachment_id=%d", $post_id);
        $wpdb->query($sql);

        // update the attachment count
        if ( !empty( $folders_to_edit ) ) {
            $ids = implode( ',', array_map( 'intval', $folders_to_edit ) );
            $sql = $wpdb->prepare("
                UPDATE {$tableFolders} AS F
                SET count = (SELECT COUNT(folder_id) FROM {$tableAttachments} AS A WHERE A.folder_id = F.id)
                WHERE id IN(%1s)",
                $ids
            );
            $wpdb->query($sql);
        }
    }

    public function addAttachment( $attachment_id ) {
        if ( isset( $_REQUEST['folder'] ) ) {
            $folder_id = intval( $_REQUEST['folder'] );

            global $wpdb;
            $tableFolders = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_folders';
            $tableAttachments = $wpdb->prefix . WPCOMMANDER_PLUGIN_DB_TABLE_PREFIX . '_attachments';

            // add new attachments
            $wpdb->insert(
                $tableAttachments,
                [
                    'folder_id' => $folder_id,
                    'attachment_id' => $attachment_id
                ]
            );

            // update the attachment count
            $sql = $wpdb->prepare("
                UPDATE {$tableFolders} AS F
                SET count = (SELECT COUNT(folder_id) FROM {$tableAttachments} AS A WHERE A.folder_id = F.id)
                WHERE id=%d",
                $folder_id
            );
            $wpdb->query( $sql );
        }
    }

    public function adminHead() {
        echo '<style>#screen-meta-links {position: absolute; right: 0;} .wrap {margin-top: 15px;}</style>';
    }

    private function getGlobals() {
        $type = FoldersModel::getCurrentType();
        $meta = UserModel::getMeta( $type );
        $config = ConfigModel::get();

        $globals = [
            'data' => [
                'version' => WPCOMMANDER_PLUGIN_VERSION,
                'type' => $type,
                'meta' => $meta,
                'default_color' => $config['default_color'],
                'disable_counter' => $config['disable_counter'],
                'disable_ajax' => $config['disable_ajax']
            ],
            'msg' => HelperModel::getMessagesForSidebar(),
            'api' => [
                'nonce' => wp_create_nonce( 'wp_rest' ),
                'url' => esc_url_raw( rest_url( WPCOMMANDER_PLUGIN_REST_URL ) )
            ]
        ];

        return $globals;
    }
}